package octoshape.android.stream;

import android.content.Context;
import android.os.Build;
import android.util.Log;

import com.google.android.gms.cast.MediaInfo;
import com.google.android.gms.cast.MediaStatus;
import octoshape.android.CastApplication;
import com.google.android.libraries.cast.companionlibrary.cast.VideoCastManager;
import com.google.android.libraries.cast.companionlibrary.cast.callbacks.VideoCastConsumerImpl;
import com.google.android.libraries.cast.companionlibrary.cast.exceptions.NoConnectionException;
import com.google.android.libraries.cast.companionlibrary.cast.exceptions.TransientNetworkDisconnectionException;
import com.octoshape.android.client.OctoStatic;
import com.octoshape.android.client.OctoshapePortListener;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.HashMap;

import octoshape.osa2.android.OctoshapeSystem;
import octoshape.osa2.listeners.OctoshapeSystemListener;
import octoshape.osa2.listeners.ProblemListener;

/**
 * Handles the creation of the streams, and the communication between the media player and stream player.
 * Created by dota on 13-02-2015.
 */
public class StreamManager {

    public static final String TAG = "StreamManager";
    public static final String MPL_TAG = "sm_NotifyStreamState";
    public static final String STREAM_ADDR = "stream";

    private static final boolean USE_AUTHENTICATON = false;
    private static boolean connected = false;
    private static String authId = null;
    private final VideoCastManager mCastManager;

    /**
     * Map from an octoshape:// link to the associated {@linkplain octoshape.android.stream.StreamPlayerWrapper}
     * The association is created when the {@linkplain octoshape.android.stream.StreamPlayerWrapper} is constructed
     */
    private HashMap<String, StreamPlayerWrapper> stream2WrapperMap = new HashMap<String, StreamPlayerWrapper>();

    private OctoshapeSystem os;

    private String lastPlayedStream = null;
    private PlaybackState lastPlaybackState = null;


    static enum PlaybackState {
        PLAYING, FINISHED, SEEKING;
    }

    StreamManager(Context context, ProblemListener problemListener, OctoshapePortListener portListener) {
        Log.d(TAG, "Stream Manager initializing ...");

        os = OctoStatic.create(context, problemListener, portListener);

        os.setOctoshapeSystemListener(new OctoshapeSystemListener() {
            // called once the Octoshape service/client has started.
            @Override
            public void onConnect(String authId) {
                StreamManager.connected = true;
                StreamManager.authId = authId;

                Log.d(TAG, "Connected to the Octoshape System");
            }
        });

        // Adding AndroidMediaPlayer
        os.addPlayerNameAndVersion(OctoshapeSystem.MEDIA_PLAYER_NATIVE, OctoshapeSystem.MEDIA_PLAYER_NATIVE,"" + Build.VERSION.SDK_INT);
        os.open();

        mCastManager = CastApplication.getCastManager();
        mCastManager.addVideoCastConsumer(new VideoCastConsumerImpl() {

            @Override
            public void onRemoteMediaPlayerStatusUpdated(){
                String stream = null;
                PlaybackState state = null;
                try {
                    stream = CastApplication.getCastManager().getRemoteMediaInformation().getMetadata().getString(STREAM_ADDR);
                } catch (TransientNetworkDisconnectionException e) {
                    e.printStackTrace();
                    return;
                } catch (NoConnectionException e) {
                    e.printStackTrace();
                    return;
                } catch (NullPointerException e) {
                    e.printStackTrace();
                    return;
                }

                int status = CastApplication.getCastManager().getPlaybackStatus();

                String result = "";

                switch (status) {
                    case MediaStatus.PLAYER_STATE_BUFFERING:
                        result = "Player state: BUFFERING";
                        break;
                    case MediaStatus.PLAYER_STATE_PAUSED:
                        result = "Player state: PAUSED";
                        break;
                    case MediaStatus.PLAYER_STATE_PLAYING:
                        result = "Player state: PLAYING";
                        state = PlaybackState.PLAYING;
                        break;
                    case MediaStatus.PLAYER_STATE_IDLE:
                        int reason = CastApplication.getCastManager().getIdleReason();
                        switch (reason) {
                            case MediaStatus.IDLE_REASON_CANCELED:
                                result = "Player state: IDLE. Reason: CANCELLED";
                                break;
                            case MediaStatus.IDLE_REASON_ERROR:
                                result = "Player state: IDLE. Reason: ERROR";
                                state = PlaybackState.FINISHED;
                                break;
                            case MediaStatus.IDLE_REASON_FINISHED:
                                result = "Player state: IDLE. Reason: FINISHED";
                                state = PlaybackState.FINISHED;
                                break;
                            case MediaStatus.IDLE_REASON_INTERRUPTED:
                                result = "Player state: IDLE. Reason: INTERRUPTED";
                                break;
                            case MediaStatus.IDLE_REASON_NONE:
                                result = "Player state: IDLE. Reason: NONE";
                                break;
                        }
                        break;
                    default:
                        result = "Player state: UNKNOWN";
                }

                Log.d(MPL_TAG, "Remote player changed: " + result);

//                if (stream != null && state != null) {
//                    mediaPlayerStatusChanged(stream, state);
//                }
            }
        });
    }

    public static StreamManager initializeStreamManager(Context context, ProblemListener problemListener, OctoshapePortListener portListener) {
        return new StreamManager(context, problemListener, portListener);
    }

    public boolean isConnected() {
        return connected;
    }

    public void addListener(String stream, IStreamPlayerWrapperListener listener) {
        StreamPlayerWrapper streamPlayerWrapper = stream2WrapperMap.get(stream);
        if (streamPlayerWrapper != null) {
            streamPlayerWrapper.setListener(listener);
        }
    }

    public void removeListener(String stream, IStreamPlayerWrapperListener listener) {
        StreamPlayerWrapper streamPlayerWrapper = stream2WrapperMap.get(stream);
        if (streamPlayerWrapper != null) {
            streamPlayerWrapper.removeListener(listener);
        }
    }

    /**
     * Adding a stream to the OctoMediaPlayer. Creates an StreamPlayer the
     * StreamPlayer instance is initiated with it's own UrlListener and
     * ProblemListener.
     *
     * @return StreamPlayer instance allowing playback
     */

    public void setupStream(final MediaInfo mediaInfo, IStreamPlayerWrapperListener listener) {

        String stream = StreamManager.getStreamFromMediaInfo(mediaInfo);
        StreamPlayerWrapper spw = stream2WrapperMap.get(stream);

        if (spw == null) {
            Log.d(TAG, "Setting up stream: " + stream);

            spw = new StreamPlayerWrapper(os.createStreamPlayer(stream), stream, mediaInfo);

            if (USE_AUTHENTICATON) {
                try {
                    spw.authenticateStreamPlayer(authId);
                } catch (IOException e) {
                    Log.e(TAG, "Error getting authentication token from server", e);
                }
            }

            stream2WrapperMap.put(stream, spw);
        } else {
            Log.d(TAG, "Stream existing for: " + stream);
        }

        addListener(stream, listener);
        spw.requestPlay();
    }

    private void mediaPlayerStatusChanged(String stream, PlaybackState newState) {
        if (lastPlayedStream != null && !stream.equals(lastPlayedStream) && newState != PlaybackState.FINISHED) {
            notifyStreamWrapper(lastPlayedStream, PlaybackState.FINISHED);
        }

        if (lastPlayedStream != null && stream.equals(lastPlayedStream) && newState == lastPlaybackState) {
            Log.d(TAG, "Double fire state " + newState);
        } else {
            notifyStreamWrapper(stream, newState);
        }

        lastPlayedStream = stream;
        lastPlaybackState = newState;
    }

    private void notifyStreamWrapper(String stream, PlaybackState newState) {
        Log.d(MPL_TAG, "Notify Media player listener. STATE: " + newState + ": " + stream);
        StreamPlayerWrapper spw = stream2WrapperMap.get(stream);

        if (spw != null) {
            spw.mediaPlayerStatusChanged(newState);
        }

    }

    /**
     * Notify the {@linkplain octoshape.android.stream.StreamPlayerWrapper} the playback on the given stream started
     * @param stream
     */
    public void playbackStarted(String stream) {
        mediaPlayerStatusChanged(stream, PlaybackState.PLAYING);
    }

    /**
     * Notify the {@linkplain octoshape.android.stream.StreamPlayerWrapper} the playback on the given stream finished
     * @param stream
     */
    public void playbackFinished(String stream) {
        mediaPlayerStatusChanged(stream, PlaybackState.FINISHED);
    }

    /**
     * Notify the {@linkplain octoshape.android.stream.StreamPlayerWrapper} the given stream will do a seek. This is important, since the
     * media player might stop playing, and we do not want to close the stream in this event.
     * @param stream
     */
    public void playbackSeeking(String stream) {
        mediaPlayerStatusChanged(stream, PlaybackState.SEEKING);
    }

    /**
     * Perform the seek in case of {@linkplain octoshape.android.stream.SeekType#OSA_SEEK}
     * @param stream
     * @param progress
     */
    public void seek(String stream, int progress ) {
        StreamPlayerWrapper spw = stream2WrapperMap.get(stream);

        if (spw != null) {
            spw.seek(progress);
        }
    }

    /**
     * Retrieves the octoshape:// stream address from the media info
     * @param mediaInfo
     * @return
     */
    public static String getStreamFromMediaInfo(MediaInfo mediaInfo) {
        JSONObject customData = mediaInfo.getCustomData();
        String result = null;

        if (customData != null) {
            try {
                result = customData.getString(StreamManager.STREAM_ADDR);
            } catch (JSONException e) {
                e.printStackTrace();
            }
        }

        Log.d(TAG, "Stream address: " + result);

        return result;
    }
}

